/*	
    $Id: SharedMem.h, v 1.0 29/May/2005$

	Author:        Leo Reyes
    E-mail:        lreyes013@gmail.com
    Copyright (C): 2004, Leo Reyes
	  
	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <windows.h>
#include <string.h>
#include <iostream>

using namespace std;

#include "SharedMem.h"

SharedMem::SharedMem(char *Name, int Size) {
  strcpy(SharedMemName, Name);
  SharedMemSize = Size;  
  MemAvailable = false;
  if (Size <= 0) return;
  
  hMapFile = CreateFileMapping(INVALID_HANDLE_VALUE, // current file handle, not backed up by any file
        NULL,                              // Default security 
        PAGE_READWRITE,                    // Read/Write permission 
        0,                                 // Hi-dword of size
        SharedMemSize,                     // Lo-dword of size
        SharedMemName);                    // Name of memory block
 
    if (hMapFile == NULL) { 
        cerr << "Could not create file mapping object." << endl;
        MemAvailable = false;
        return;
    }
    cerr << "Shared memory object succesfully opened" << endl;    
    
    MemPtr = MapViewOfFile(hMapFile, // handle to mapping object 
        FILE_MAP_ALL_ACCESS,               // read/write permission 
        0,                                 // Hi-Dword file offset
        0,                                 // Lo-Dword file offset
        0);                                // Size of block: 0 = Map entire file 
 
     if (MemPtr == NULL) { 
         cerr << "Could not map view of file." << endl;
         MemAvailable = false;
         return;
     }
     cerr << "Memory view opened" << endl;
     MemAvailable = true;
     
     // Create the read-write mutex:
     hRWMutex = CreateMutex(NULL, // No security attributes
          FALSE,                  // Don't acquire ownership initially
          "SharedMemRWMutex"         // Name
     );
}

SharedMem::~SharedMem() {
  // Destroy the event
  CloseHandle( hRWMutex );
  
  BOOL Res = UnmapViewOfFile(MemPtr);
  if (Res == 0) {
      //cerr << "Error, could not unmap view of file" << endl;
      return;
  }
  //cerr << "File view unmapped" << endl;
  
  if (CloseHandle(hMapFile) == 0) {
      //cerr << "Error, could not close handle to shared memory object" << endl;
      return;
  }
  cerr << "Shared memory object closed" << endl;
}

int SharedMem::GetSize() {
  return SharedMemSize;
}

char *SharedMem::GetName() {
  return SharedMemName;
}

void *SharedMem::GetMemory() {
  return MemPtr;
}

void SharedMem::Write(void *Src, int Bytes) {
  WaitForSingleObject(hRWMutex, INFINITE); // After the wait is complete, this process
  // automatically acquires the mutex, excluding the rest
  memcpy(MemPtr, Src, Bytes); // Write
  ReleaseMutex(hRWMutex);     // Release ownership
}

void SharedMem::Read(void *Dest, int Bytes) {
  WaitForSingleObject(hRWMutex, INFINITE); // Request ownership of the mutex
  memcpy(Dest, MemPtr, Bytes); // Read
  ReleaseMutex(hRWMutex);      // Release ownership
}
